#####################################################################
# NAME:  Chris Bilder                                               #
# Purpose: Larry Bird contingency table analysis                    #
#                                                                   #
# NOTES:                                                            #
#####################################################################

#####################################################################
# Data

  n.table<-array(data = c(251, 48, 34, 5), dim = c(2,2), dimnames = list(First = c("made", "missed"),
             Second = c("made", "missed")))
  n.table
  class(n.table)
  n.table[1,1]
  n.table[1,]

  #Estimated odds ratio
  theta.hat<-n.table[1,1]*n.table[2,2]/(n.table[1,2]*n.table[2,1])
  theta.hat
  1/theta.hat

  #Pearson chi-square test
  ind.test<-chisq.test(n.table, correct = FALSE)
  ind.test
  names(ind.test)
  #just p-value
  ind.test$p.value

  #Exact test
  chisq.test(n.table, correct = FALSE, simulate.p.value = TRUE, B = 1000)


######################################################################
#Convert contigency table data to a data.frame

  #Convert to a data.frame
  bird.df<-as.data.frame(as.table(n.table))
  bird.df
  
  mod.fit<-glm(formula = Freq ~ First + Second, data = bird.df, family = poisson(link = log))
  summary(mod.fit)
  
  #LRT for independence
  1 - pchisq(q = mod.fit$deviance, df = mod.fit$df.residual)
  
  #Convert back to an array
  n.table2<-xtabs(Freq ~ First + Second, data = bird.df)
  n.table2


######################################################################
#What if the data did not already come in a contingency table format?

  #Create "raw" data
  miss.miss<-matrix(rep(x = c("missed", "missed"), times = 5), nrow = 5, ncol = 2, byrow=T)
  miss.make<-matrix(rep(x = c("missed", "made"), times = 48), nrow = 48, ncol = 2, byrow=T)
  make.miss<-matrix(rep(x = c("made", "missed"), times = 34), nrow = 34, ncol = 2, byrow=T)
  make.make<-matrix(rep(x = c("made", "made"), times = 251), nrow = 251, ncol = 2, byrow=T)

  #Put "raw" data into one data.frame
  all.data<-rbind(miss.miss, miss.make, make.miss, make.make)
  all.data2<-data.frame(all.data)

  #Gives new names to columns
  names(all.data2)<-c("first", "second")

  #Find contingency table two different ways
  bird.table1<-table(all.data2$first, all.data2$second)
  bird.table1

  bird.table2<-xtabs(formula = ~ first + second, data = all.data2)
  bird.table2

  chisq.test(bird.table1, correct = FALSE)
  chisq.test(first, second, data = all.data, correct = FALSE)







#