############################################################################
# NAME:  Chris Bilder                                                      #
# DATE:  8-14-11                                                           #
# PURPOSE: Data analysis example in R using the gpa data set               #
#                                                                          #
# NOTES:                                                                   #
#                                                                          #
############################################################################

############################################################################
# Data management
    
  #Read in the data with spaces separating variable values
  gpa<-read.table(file = "C:\\chris\\unl\\Dropbox\\NEW\\workshop\\Gallup\\gpa.txt", header = TRUE, sep = "")
 
  #Print data set
  gpa
  head(gpa)

  #Read in the data with commas separating variable values
  gpa.comma1<-read.table(file = "C:\\chris\\unl\\Dropbox\\NEW\\workshop\\Gallup\\gpa.csv", header = TRUE, sep = ",")
  head(gpa.comma1)
  #Another way
  gpa.comma2<-read.csv(file = "C:\\chris\\unl\\Dropbox\\NEW\\workshop\\Gallup\\gpa.csv", header = TRUE)
  head(gpa.comma2)

  #Exporting data as a comma delimited file
  write.csv(x = gpa, file = "C:\\chris\\unl\\Dropbox\\NEW\\workshop\\Gallup\\temp.csv")

  #Read in an Excel version of the file
  library(package = RODBC)
  z<-odbcConnectExcel(xls.file = "C:\\chris\\unl\\Dropbox\\NEW\\workshop\\Gallup\\gpa.xls")
  gpa.excel<-sqlFetch(channel = z, sqtable = "sheet1")
  close(z)

  head(gpa.excel)

  #Summary statistics for variables
  summary(gpa)

  #Access variable values
  names(gpa)
  gpa$HS.GPA
  gpa[1,1]
  gpa[,1]
  gpa[1,1:2]
  gpa[1,c(1,2)]
  gpa[,-2]
  
  #Accessing observations based on a condition
  gpa$HS.GPA<2.5
  gpa[gpa$HS.GPA<2.5, ]
  sum(gpa$HS.GPA<2.5)

  #If then else - note that "&" means "and"
  test.cond<-ifelse(test = gpa$HS.GPA < 2.5 & gpa$College.GPA < 2.5, yes = 1, no = 0)
  sum(test.cond)

  #If then else - note that "|" means "or"
  test.cond<-ifelse(test = gpa$HS.GPA < 2.5 | gpa$College.GPA < 2.5, yes = 1, no = 0)
  sum(test.cond)


############################################################################
# Scatter plot

  #Simple plot
  plot(x = gpa$HS.GPA, y = gpa$College.GPA)

  #Better plot
  plot(x = gpa$HS.GPA, y = gpa$College.GPA, xlab = "HS GPA", ylab = "College GPA", main = "College GPA vs. HS GPA",
     xlim = c(0,4.5), ylim = c(0,4.0), col = "red", pch = 1, cex = 1.0, lwd = 2,
     panel.first = grid(col = "gray", lty = "dotted"))

  #Controlling the x and y axis tick marks
  plot(x = gpa$HS.GPA, y = gpa$College.GPA, xlab = "HS GPA", ylab = "College GPA", main = "College GPA vs. HS GPA",
     xlim = c(0,4.5), ylim = c(0,4.0), col = "red", pch = 1, cex = 1.0, lwd = 2,
     panel.first=grid(col = "gray", lty = "dotted"), xaxt = "n")
  #Major tick marks for x-axis
  axis(side = 1, at = seq(from = 0, to = 4.5, by = 0.5))
  #Minor tick marks for x-axis
  axis(side = 1, at = seq(from = 0, to = 4.5, by = 0.1), tck = 0.01, labels = FALSE)


########################################################################
# Fitting the model
     
  #Fit the simple linear regression model and save the results in mod.fit
  mod.fit<-lm(formula = College.GPA ~ HS.GPA, data = gpa)

  #A very brief look of what is inside of mod.fit - see the summary function for a better way
  mod.fit

  #See the names of all of the object components
  names(mod.fit)
  mod.fit$coefficients 
  mod.fit$residuals

  #Put some of the components into a data.frame object
  save.fit<-data.frame(gpa, College.GPA.hat = round(mod.fit$fitted.values,2), residuals = round(mod.fit$residuals,2))

  #Print contents of save.fit
  head(save.fit)

  #Summarize the information stored in mod.fit
  summary(object = mod.fit)
  
  #########################################
  #Categorical independent variable example
  where.live<-c("with parents", "dorm", "off-campus")
  x<-rep(x = where.live, each = 7)
  gpa2<-data.frame(gpa, where.live = x[-21])
  head(gpa2)
  levels(gpa2$where.live)
  contrasts(gpa2$where.live)

  #Fit model with where.live
  mod.fit2<-lm(formula = College.GPA ~ HS.GPA + where.live, data = gpa2)
  summary(mod.fit2)
  
  #Provides a test of the entire categorical variable
  anova(object = mod.fit2, test = "F")

  #What if categorical variable was numerical?
  where.live<-c(3, 1, 2)
  x<-rep(x = where.live, each = 7)
  gpa3<-data.frame(gpa, where.live = x[-21])
  head(gpa3)

  #Fit model where where.live is a numerically coded categorical variable
  mod.fit3<-lm(formula = College.GPA ~ HS.GPA + factor(where.live), data = gpa3)
  summary(mod.fit3)

  #Provides a test of the entire categorical variable
  anova(object = mod.fit3, test = "F")


  #Transformations of independent variable - Square term
  mod.fit4<-lm(formula = College.GPA ~ HS.GPA + I(HS.GPA^2), data = gpa)
  summary(mod.fit4)


########################################################################
# Object oriented language

  #Classes of various objects seen so far
  class(gpa)
  class(gpa$HS.GPA)
  class(lm)
  class(mod.fit)
  
  methods(class = lm) #Method functions associated with lm()
  methods(generic.function = summary) #Generic functions associated with summary()

  #Other examples of using generic functions with mod.fit
  anova(object = mod.fit)
  vcov(object = mod.fit)
  confint(object = mod.fit, level = 0.95)
  residuals(object = mod.fit)
  AIC(object = mod.fit)
  rstudent(model = mod.fit)
  influence(model = mod.fit)
  plot(x = mod.fit)


########################################################################
#Estimating the response

  #Open a new graphics window
  win.graph(width = 6, height = 6, pointsize = 10)

  #Same scatter plot as before
  plot(x = gpa$HS.GPA, y = gpa$College.GPA, xlab = "HS GPA", ylab = "College GPA", main = "College GPA vs. HS GPA",
       xlim = c(0,4.5), ylim = c(0,4.5), col = "red", pch = 1, cex = 1.0, panel.first = grid(col = "gray", lty = "dotted"))

  #Puts the line y = a + bx on the plot
  abline(a = mod.fit$coefficients[1], b = mod.fit$coefficients[2], lty = "solid", col = "blue", lwd = 2)


  #Notice the above line goes outside of the range of the x-values. To prevent this, we can use the segments function
  #Open a new graphics window - do not need to
  win.graph(width = 6, height = 6, pointsize = 10)

  #Same scatter plot as before
  plot(x = gpa$HS.GPA, y = gpa$College.GPA, xlab = "HS GPA", ylab = "College GPA", main = "College GPA vs. HS GPA",
       xlim = c(0,4.5), ylim = c(0,4.5), col = "red", pch = 1, cex = 1.0, panel.first = grid(col = "gray", lty = "dotted"))

  #Draw a line from (x0, y0) to (x1, y1)
  segments(x0 = min(gpa$HS.GPA), y0 = mod.fit$coefficients[1] + mod.fit$coefficients[2]*min(gpa$HS.GPA),
           x1 = max(gpa$HS.GPA), y1 = mod.fit$coefficients[1] + mod.fit$coefficients[2]*max(gpa$HS.GPA),
           lty = "solid", col = "blue", lwd = 2)


  #Predictions
  pred.data<-data.frame(HS.GPA = c(2, 3, 4))
  predict(object = mod.fit, newdata = pred.data)
  predict(object = mod.fit, newdata = pred.data, se.fit = TRUE, interval = "confidence", level = 0.95)
  save.pred<-predict(object = mod.fit, newdata = pred.data, se.fit = TRUE, interval = "confidence", level = 0.95)
  names(save.pred)
  save.pred$fit

  #Same scatter plot as before
  plot(x = gpa$HS.GPA, y = gpa$College.GPA, xlab = "HS GPA", ylab = "College GPA", main = "College GPA vs. HS GPA",
       xlim = c(0,4.5), ylim = c(0,4.5), col = "red", pch = 1, cex = 1.0, panel.first = grid(col = "gray", lty = "dotted"))
  curve(expr = predict(object = mod.fit, newdata = data.frame(HS.GPA = x)), from = min(gpa$HS.GPA),
    to = max(gpa$HS.GPA), add = TRUE, n = 1000)
  curve(expr = predict(object = mod.fit, newdata = data.frame(HS.GPA = x), se.fit = TRUE, interval = "confidence", level = 0.95)$fit[,2],
    from = min(gpa$HS.GPA), to = max(gpa$HS.GPA), add = TRUE, col = "blue", lty = "dashed")
  curve(expr = predict(object = mod.fit, newdata = data.frame(HS.GPA = x), se.fit = TRUE, interval = "confidence", level = 0.95)$fit[,3],
    from = min(gpa$HS.GPA), to = max(gpa$HS.GPA), add = TRUE, col = "blue", lty = "dashed")
  legend(x = 1, y = 4, legend = c("Estimated college GPA", "95% confidence interval"), lty = c("solid", "dashed"),
    col = c("black", "blue"), bty = "n")
  #legend(locator(1), legend = c("Estimated college GPA", "95% confidence interval"), lty = c("solid", "dashed"), col = c("black", "red"), bty = "n")
  #identify(x = gpa$HS.GPA, y = gpa$College.GPA)


  #How to put mathematical symbols on plot
  plot(x = gpa$HS.GPA, y = gpa$College.GPA, xlab = "HS GPA", ylab = "College GPA",
      main = expression(paste("College GPA vs. HS GPA and ", widehat(College.GPA) == hat(beta)[0] + hat(beta)[1]*HS.GPA)),
      xlim = c(0,4.5), ylim = c(0,4.5), col = "red", pch = 1, cex = 1.0, panel.first=grid(col = "gray", lty = "dotted"))
  #Draw a line from (x0, y0) to (x1, y1)
  segments(x0 = min(gpa$HS.GPA), y0 = mod.fit$coefficients[1] + mod.fit$coefficients[2]*min(gpa$HS.GPA),
           x1 = max(gpa$HS.GPA), y1 = mod.fit$coefficients[1] + mod.fit$coefficients[2]*max(gpa$HS.GPA),
           lty = 1, col = "blue", lwd = 2)
  demo(plotmath) #Run this to see examples


############################################################################################################
# Create a function to find the estimated simple linear regression model and put the line on a scatter plot

  my.reg.func<-function(x, y, data) {
  
    #Fit the simple linear regression model and save the results in mod.fit
    mod.fit<-lm(formula = y ~ x, data = data)
  
    #Open a new graphics window
    win.graph(width = 6, height = 6, pointsize = 10)
 
    #Same scatter plot
    plot(x = x, y = y, xlab = "x", ylab = "y", main = "y vs. x", 
         col = "red", pch = 1, cex = 1.0, panel.first=grid(col = "gray", lty = "dotted"))

    #Draw a line from (x0, y0) to (x1, y1)
    segments(x0 = min(x), y0 = mod.fit$coefficients[1] + mod.fit$coefficients[2]*min(x), 
             x1 = max(x), y1 = mod.fit$coefficients[1] + mod.fit$coefficients[2]*max(x),
             lty = 1, col = "blue", lwd = 2)
    
    #This is the object returned
    mod.fit           
  }


  save.it<-my.reg.func(x = gpa$HS.GPA, y = gpa$College.GPA, data = gpa)  
  names(save.it)
  summary(save.it)
  

  #Model diagnostics
  #   The examine.model.simple() function is in examine.model.simple.R. This code needs to be run first
  #     before using the function. Using source() (with the correct file location) will do this; otherwise
  #     open the program in an editor and run it from there.
  #   Note that you need to have the car and lmtest packages installed if const.var.test = TRUE
  source("C:\\chris\\unl\\Dropbox\\NEW\\workshop\\Gallup\\examine.model.simple.R")
  save.it<-examine.model.simple(mod.fit.obj = mod.fit, const.var.test = TRUE, boxcox.find = TRUE)

  names(save.it)
  save.it$sum.data
  save.it$levene


############################################################################
# Graphics

  #Histograms
  par(mfrow = c(2,1)) #Two rows and one column of plots
  hist(x = gpa$HS.GPA, xlab = "HS GPA", main = "Histogram of HS GPA", breaks = c(0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5))
  hist(x = gpa$College.GPA, xlab = "College GPA", main = "Histogram of College GPA", breaks = seq(from = 0, to = 4.5, by = 0.5))

  #Same plot but with density overlays
  hist(x = gpa$HS.GPA, xlab = "HS GPA", main = "Histogram of HS GPA", breaks = c(0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5), freq = FALSE)
  curve(expr = dnorm(x = x, mean = mean(gpa$HS.GPA), sd = sd(gpa$HS.GPA)), col = "red", add = TRUE)
  hist(x = gpa$College.GPA, xlab = "College GPA", main = "Histogram of College GPA", breaks = seq(from = 0, to = 4.5, by = 0.5), freq = FALSE)
  curve(expr = dnorm(x = x, mean = mean(gpa$College.GPA), sd = sd(gpa$College.GPA)), col = "red", add = TRUE)

  #Box and dot plots overlaid
  par(mfrow = c(1,1))
  boxplot(x = gpa, col = "lightblue", main = "Box and dot plots", ylab = "GPA",  xlab = "")
  stripchart(x = gpa, lwd = 2, col = "red", method = "jitter", vertical = TRUE, pch = 1, add = TRUE)

  #Below is a different form of the data which requires a different way to specify what to plot
  HS.only<-data.frame(school = "HS", gpa = gpa$HS.GPA)
  College.only<-data.frame(school = "College", gpa = gpa$College.GPA)
  HS.college<-rbind(HS.only, College.only)
  head(HS.college)
  tail(HS.college)
  boxplot(formula = gpa ~ school, data = HS.college, col = "lightblue", main = "Box and dot plots", ylab = "GPA",  xlab = "")
  stripchart(x = HS.college$gpa ~ HS.college$school, lwd = 2, col = "red", method = "jitter", vertical = TRUE, pch = 1, add = TRUE)

  #Quick example with lattice package
  library(package = lattice)
  xyplot(x = College.GPA ~ HS.GPA, data = gpa, main = "College GPA vs. HS GPA")



#
